#ifndef __dropsource_inc__
#define __dropsource_inc__

#ifndef __ole2_inc__
#include "ole2.inc"
#endif

#ifndef __unknown_inc__
#include "unknown.inc"
#endif

// use CDropSource as the main class
// call CDropSource::m_dataobject.InitNew() before adding items
// call CDropSource::m_dataobject.AddData() to add data, at least once
// call CDropSource::m_dataobject.m_enum.Reset(); to initialize enumerator
// call DoDragDrop(&m_lvDropSource.m_dataobject, &m_lvDropSource,...) to begin drag-drop
//      where m_lvDropSource represents CDropSource instance


// CEnumFORMATETC::Next returns lindex always set to -1

struct DATAOBJECT // allocated by single new() with data
{
	DATAOBJECT *next;
	FORMATETC format;
	int       DataSize;

	HBITMAP       hBitmap[0];
	HMETAFILEPICT hMetaFilePict[0];
	HENHMETAFILE  hEnhMetaFile[0];
	HGLOBAL       hGlobal[0];
	LPWSTR        lpszFileName[0];
	IStream      *pstm[0];
	IStorage     *pstg[0];
	byte          data[0];
}


class CEnumFORMATETC : CUnknown
{
	declare  CEnumFORMATETC();
	declare _CEnumFORMATETC();
	declare virtual QueryInterface(REFIID *riid,void *ppvObject),HRESULT;
	declare virtual Next(ULONG celt,/*out*/FORMATETC *rgelt,/*out*/ULONG *pceltFetched),HRESULT;
	declare virtual Skip(ULONG celt),HRESULT;
	declare virtual Reset(),HRESULT;
	declare virtual Clone(/*out*/IEnumFORMATETC *ppenum),HRESULT;

	declare Add(DATAOBJECT *node);
	declare Lookup(FORMATETC *pformatetc, DATAOBJECT *ppv),HRESULT;
	declare RemoveAll();
	DATAOBJECT *m_list; // DATAOBJECT
	DATAOBJECT *m_pos;
}


class CDataObject : CUnknown
{
	declare virtual QueryInterface(REFIID *riid,void *ppvObject),HRESULT;
	declare virtual GetData(FORMATETC *pformatetcIn,/*out*/STGMEDIUM *pmedium),HRESULT;
	declare virtual GetDataHere(FORMATETC *pformatetc,/*out*/STGMEDIUM *pmedium),HRESULT;
	declare virtual QueryGetData(FORMATETC *pformatetc),HRESULT;
	declare virtual GetCanonicalFormatEtc(FORMATETC *pformatectIn,/*out*/FORMATETC *pformatetcOut),HRESULT;
	declare virtual SetData(FORMATETC *pformatetc,STGMEDIUM *pmedium,BOOL fRelease),HRESULT;
	declare virtual EnumFormatEtc(DWORD dwDirection,/*out*/IEnumFORMATETC *ppenumFormatEtc),HRESULT;
	// advise not implemented
	declare virtual DAdvise(FORMATETC *pformatetc,DWORD advf,IAdviseSink *pAdvSink,/*out*/DWORD *pdwConnection),HRESULT;
	// advise not implemented
	declare virtual DUnadvise(DWORD dwConnection),HRESULT;
	// advise not implemented
	declare virtual EnumDAdvise(/*out*/IEnumSTATDATA *ppenumAdvise),HRESULT;

	// remove provious data
	declare InitNew();
	// private: returns pointer to DATAOBJECT structure if
	// given FORMATETC was previously added
	declare FindFormat(FORMATETC *pformatetc),DATAOBJECT *;

	// if pformatetc->tymed != TYMED_HGLOBAL, DataSize should be 4
	declare AddData(FORMATETC *pformatetc,void *pData, int DataSize),HRESULT;

	CEnumFORMATETC m_enum;
}


class CDropSource : CUnknown
{
	declare virtual QueryInterface(REFIID *riid,void *ppvObject),HRESULT;
	declare virtual QueryContinueDrag(BOOL fEscapePressed, DWORD grfKeyState),HRESULT;
	declare virtual GiveFeedback(DWORD dwEffect),HRESULT;
	CDataObject m_dataobject;
}

#endif
